Attribute VB_Name = "Main"
Option Explicit
Option Private Module

Public Sub CC_RunGroupByRank()
  Dim iSelection As Excel.Range: Set iSelection = ActiveWorkbook.Application.Selection
  Dim nRow&: nRow = 1
  Dim sTxt$
  Do
    sTxt = iSelection.Cells(nRow, 1)
    If sTxt = vbNullString Then _
      Exit Do
    If IsNumeric(sTxt) Then _
      iSelection.Rows(nRow).OutlineLevel = CLng(sTxt)
    nRow = nRow + 1
  Loop
End Sub

Public Sub CC_RunGroupByID()
  Dim iSelection As Excel.Range: Set iSelection = ActiveWorkbook.Application.Selection
  Dim nRow&: nRow = 1
  Dim lastID$: lastID = vbNullString
  Dim curID$
  Do
    curID = iSelection.Cells(nRow, 1)
    If curID = vbNullString Then _
      Exit Do
    iSelection.Rows(nRow).OutlineLevel = IIf(lastID = curID, 2, 1)
    lastID = curID
    nRow = nRow + 1
  Loop
End Sub

Public Sub CC_RunClearFormatting()
  Dim iSelection As Excel.Range: Set iSelection = ActiveWorkbook.Application.Selection
  Do While iSelection.FormatConditions.Count > 0
    Call iSelection.FormatConditions.Item(1).Delete
  Loop
End Sub

Public Sub CC_RunTrimCells()
  Call XLForEachNonEmptyCell( _
    target:=ActiveWorkbook.Application.Selection, _
    oCallback:=New CellProcessor, _
    sFuncName:="TrimCell")
End Sub

Public Sub CC_RunTrimPunctuation()
  Call XLForEachNonEmptyCell( _
    target:=ActiveWorkbook.Application.Selection, _
    oCallback:=New CellProcessor, _
    sFuncName:="TrimCellPunctuation")
End Sub

Public Sub CC_RunSubstituteWhitespace()
  Call XLForEachNonEmptyCell( _
    target:=ActiveWorkbook.Application.Selection, _
    oCallback:=New CellProcessor, _
    sFuncName:="SubstituteWS")
End Sub

Public Sub CC_RunLinksCreate()
  Call XLForEachNonEmptyCell( _
    target:=ActiveWorkbook.Application.Selection, _
    oCallback:=New CellProcessor, _
    sFuncName:="CreateHyperlink")
End Sub

Public Sub CC_RunSplitText()
  Call XLForEachNonEmptyCell( _
    target:=ActiveWorkbook.Application.Selection, _
    oCallback:=New CellProcessor, _
    sFuncName:="SplitTextIntoColumn")
End Sub

Public Sub CC_RunLinksScan()
  Dim sMask$: sMask = UserInteraction.PromptInput("   ", sInitial:="*.*")
  If sMask = vbNullString Then _
    Exit Sub
  Dim sFolder$: sFolder = UserInteraction.PromptFolder(ActiveWorkbook.Path & "\")
  If sFolder = vbNullString Then _
    Exit Sub
    
  Dim iScanner As New LinksOutput: Call iScanner.Init(ActiveWorkbook.Application.Selection.Cells(1, 1), sMask)
  Call ForEachFileRecursive(sFolder, iScanner, "ProcessFile")
    
  Call UserInteraction.ShowMessage(IM_LINKS_SCAN_OK, iScanner.countAdded_, iScanner.countAll_)
End Sub

Public Sub CC_RunLinksValidate()
  Dim iProcessor As New CellProcessor: Call iProcessor.Init
  Call XLForEachNonEmptyCell( _
    target:=ActiveWorkbook.Application.Selection, _
    oCallback:=iProcessor, _
    sFuncName:="ValidateHyperlink")
  Call UserInteraction.ShowMessage(IM_LINKS_VALIDATION, iProcessor.countInvalid_, iProcessor.countAll_)
End Sub

Public Sub CC_RunLinksServer()
  Dim iProcessor As New CellProcessor: Call iProcessor.Init
  Call XLForEachNonEmptyCell( _
    target:=ActiveWorkbook.Application.Selection, _
    oCallback:=iProcessor, _
    sFuncName:="FixServerLink")
  Call UserInteraction.ShowMessage(IM_LINKS_SERVER, iProcessor.countInvalid_, iProcessor.countAll_)
End Sub

Public Sub CC_RunLinksRelocate()
  Dim iTarget As Excel.Workbook: Set iTarget = ActiveWorkbook
  Dim iSelection As Excel.Range: Set iSelection = iTarget.Application.Selection
  If iSelection.Hyperlinks.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_SELECTION_NO_HYPERLINKS)
    Exit Sub
  End If
  
  Dim fso As New Scripting.FileSystemObject
  Dim sInitFolder$: sInitFolder = fso.GetParentFolderName(iSelection.Hyperlinks(1).Address)
  Call RelocateDlg.Init(sInitFolder)
  Call RelocateDlg.Show
  If RelocateDlg.isCancelled_ Then _
    Exit Sub
  
  Dim iProcessor As New LinksRelocator: Call iProcessor.Init(RelocateDlg.Path, RelocateDlg.Prefix, iTarget.Path, bRemoveSource:=True)
  Call XLForEachNonEmptyCell( _
    target:=ActiveWorkbook.Application.Selection, _
    oCallback:=iProcessor, _
    sFuncName:="ProcessCell")
  
  Call UserInteraction.ShowMessage(IM_LINKS_RELOCATE_OK, iProcessor.countRelocated_, iProcessor.countAll_)
End Sub

Public Sub CC_RunLinksCopyFiles()
  Dim iTarget As Excel.Workbook: Set iTarget = ActiveWorkbook
  Dim iSelection As Excel.Range: Set iSelection = iTarget.Application.Selection
  If iSelection.Hyperlinks.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_SELECTION_NO_HYPERLINKS)
    Exit Sub
  End If
  
  Dim fso As New Scripting.FileSystemObject
  Dim sInitFolder$: sInitFolder = fso.GetParentFolderName(iSelection.Hyperlinks(1).Address)
  Call RelocateDlg.Init(sInitFolder)
  Call RelocateDlg.Show
  If RelocateDlg.isCancelled_ Then _
    Exit Sub
  
  Dim iProcessor As New LinksRelocator: Call iProcessor.Init(RelocateDlg.Path, RelocateDlg.Prefix, iTarget.Path, bRemoveSource:=False)
  Call XLForEachNonEmptyCell( _
    target:=ActiveWorkbook.Application.Selection, _
    oCallback:=iProcessor, _
    sFuncName:="ProcessCell")
  
  Call UserInteraction.ShowMessage(IM_LINKS_COPY_OK, iProcessor.countRelocated_, iProcessor.countAll_)
End Sub

Public Sub CC_RunListIncome()
  Dim iProjects As Scripting.Dictionary: Set iProjects = CCListProjects(T_PRJ_INCOME)
  If iProjects Is Nothing Then _
    Exit Sub
  Call PasteProjects(Excel.ActiveWindow.ActiveCell, iProjects)
End Sub

Public Sub CC_RunListOutcome()
  Dim iProjects As Scripting.Dictionary: Set iProjects = CCListProjects(T_PRJ_OUTCOME)
  If iProjects Is Nothing Then _
    Exit Sub
  Call PasteProjects(Excel.ActiveWindow.ActiveCell, iProjects)
End Sub

Public Sub CC_RunListLeads()
  Dim iProjects As Scripting.Dictionary: Set iProjects = CCListProjects(T_PRJ_LEAD_ACTIVE)
  If iProjects Is Nothing Then _
    Exit Sub
  Call PasteProjects(Excel.ActiveWindow.ActiveCell, iProjects)
End Sub

Public Sub CC_RunListTentative()
  Dim iProjects As Scripting.Dictionary: Set iProjects = CCListProjects(T_PRJ_LEAD_TENTATIVE)
  If iProjects Is Nothing Then _
    Exit Sub
  Call PasteProjects(Excel.ActiveWindow.ActiveCell, iProjects)
End Sub

Public Sub CC_RunLinksPrint()
  Dim selRange As Excel.Range: Set selRange = ActiveWorkbook.Application.Selection
  Dim theLinks As Excel.Hyperlinks: Set theLinks = selRange.Hyperlinks
  If theLinks.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_SELECTION_NO_HYPERLINKS)
    Exit Sub
  End If
  
  Call PrintLinks(theLinks)
End Sub

Public Sub CC_RunConfig()
  Dim fso As New Scripting.FileSystemObject
  Dim sConfigPath$: sConfigPath = ConceptConfigPath
  If Not fso.FileExists(sConfigPath) Then _
    Call SetupConfig(sConfigPath)

  Dim oShell As New Shell32.Shell
  Call oShell.Open(sConfigPath)
End Sub

Public Sub CC_RunWordAddin()
  If Not ValidateNoWordOpen Then
    Call UserInteraction.ShowMessage(EM_WORD_IS_OPEN)
    Exit Sub
  End If
  
  Dim iCurrent As AddinOptions: Set iCurrent = ScanAddinOptions()
  Call WordDlg.Init(iCurrent)
  Call WordDlg.Show
  If WordDlg.isCanceled_ Then _
    Exit Sub
  
  If ApplyAddins(iCurrent, WordDlg.GetResult) Then
    Call UserInteraction.ShowMessage(IM_WORD_ADDINS_OK)
  Else
    Call UserInteraction.ShowMessage(EM_WORD_ADDINS_FAILED)
  End If
End Sub

Public Sub CC_RunUpdateTemplates()
  If Not ValidateNoWordOpen Then
    Call UserInteraction.ShowMessage(EM_WORD_IS_OPEN)
    Exit Sub
  End If
  
  If CopyFileOrFolder( _
    sSource:=SERVER_TEMPLATES_FOLDER, _
    sDestination:=VBA.Environ$("APPDATA") & "\" & LOCAL_TEMPLATES_FOLDER) Then
    Call UserInteraction.ShowMessage(IM_TEMPLATES_OK)
  Else
    Call UserInteraction.ShowMessage(EM_TEMPLATES_FAILED)
  End If
End Sub

Public Sub CC_RunUpdateConceptCore()
  Call AccessPython.UnloadDLL
  Call UnloadConceptCore
  
  If Not UpdateConceptPowershell Then
    Call UserInteraction.ShowMessage(EM_CORE_UPDATE_FAILED)
    Exit Sub
  End If
  
  Call UnregisterUDF
  Call RegisterUDF
  Call UserInteraction.ShowMessage(IM_CONCEPT_UPDATE_OK)
End Sub

Public Sub CC_RunConceptStatus()
  Dim iExcel As New API_XLWrapper
  Call iExcel.NewDocument
  
  Call OutputConceptStatus(iExcel.Document.Sheets(1))
  
  Call iExcel.Document.Activate
End Sub

Public Sub CC_RunAppdata()
  Dim oShell As New Shell32.Shell
  Call oShell.Open(VBA.Environ$("APPDATA") & "\Microsoft")
End Sub

Public Sub CC_RunAnalyseSearch()
  Dim iDoc As Excel.Workbook: Set iDoc = Excel.ActiveWorkbook
  
  Call SearchDlg.Init(iDoc)
  Call SearchDlg.Show
  If SearchDlg.isCancelled_ Then _
    Exit Sub
    
  Dim params As ParametersSearch: Set params = SearchDlg.ResultParams
  Call ExecuteSearch(iDoc, params)
  
  Call UserInteraction.ShowMessage(IM_SEARCH_OK)
End Sub

Public Sub CC_RunHelp()
  Dim iWord As New API_WordWrapper
  Dim iHelp As Word.Document: Set iHelp = iWord.OpenDocument(APP_HELP, bReadOnly:=True)
  If iHelp Is Nothing Then _
    Exit Sub

  With iHelp
    .ActiveWindow.View.ReadingLayout = False
    Call .Application.Activate
  End With
End Sub

Public Sub CC_RunDescriptionUDF()
  Call RegisterUDF
End Sub
