Attribute VB_Name = "MainImpl"
Option Explicit
Option Private Module

Public Function PasteProjects(ByVal rWhere As Excel.Range, iProjects As Scripting.Dictionary)
  If iProjects.Count = 0 Then _
    Exit Function
  
  Set rWhere = rWhere.Cells(1, 1)
  If Not IsEmpty(rWhere) Then
    Call rWhere.Offset(1).EntireRow.Insert
    Set rWhere = rWhere.Offset(1)
  End If

  Dim aKey As Variant
  For Each aKey In iProjects
    If IsEmpty(rWhere) Then _
      GoTo PASTE_PRJ
    
    Call rWhere.EntireRow.Insert
    Set rWhere = rWhere.Offset(-1)
    Call rWhere.Offset(-1).Copy(rWhere)

PASTE_PRJ:
    rWhere = CStr(aKey)
    Set rWhere = rWhere.Offset(1)
  Next aKey
End Function

Public Function PrintLinks(iLinks As Excel.Hyperlinks)
  Dim fso As New Scripting.FileSystemObject
  Dim iShell As New Shell32.Shell
  Dim aLink As Excel.Hyperlink
  For Each aLink In iLinks
    Dim sAddress$: sAddress = fso.GetAbsolutePathName(aLink.Address)
    If sAddress = vbNullString Then _
      GoTo NEXT_LINK
    If Not fso.FileExists(sAddress) Then _
      GoTo NEXT_LINK
    Call iShell.ShellExecute(sAddress, vOperation:="Print")

NEXT_LINK:
  Next aLink
End Function

Public Function SetupConfig(sPath$)
  Dim iConfig As API_Config: Set iConfig = ConceptConfig
  Call iConfig.SaveToFile(sPath)
End Function

Public Function ValidateNoWordOpen() As Boolean
  On Error Resume Next
  Dim iWord As Word.Application: Set iWord = GetObject(, "Word.Application")
  ValidateNoWordOpen = Err.Number <> 0 Or iWord Is Nothing
  On Error GoTo 0
End Function

Public Function ScanAddinOptions() As AddinOptions
  Dim fso As New Scripting.FileSystemObject
  Dim sLocals$: sLocals = VBA.Environ$("APPDATA") & "\Microsoft\Word\STARTUP"
  Set ScanAddinOptions = New AddinOptions
  With ScanAddinOptions
    .data_.Item(T_WA_CONCEPT) = fso.FileExists(sLocals & "\" & ADDIN_NAME_CONCEPT)
    .data_.Item(T_WA_NPA) = fso.FileExists(sLocals & "\" & ADDIN_NAME_NPA)
    .data_.Item(T_WA_MARKUP) = fso.FileExists(sLocals & "\" & ADDIN_NAME_MARKUP)
    .data_.Item(T_WA_PARSERS) = fso.FileExists(sLocals & "\" & ADDIN_NAME_PARSER)
    .data_.Item(T_WA_MAKET) = fso.FileExists(sLocals & "\" & ADDIN_NAME_MAKET)
  End With
End Function

Public Function ApplyAddins(iCurrent As AddinOptions, iNew As AddinOptions) As Boolean
  ApplyAddins = False
  
  Dim sParams$
  Dim nID As TWordAddin
  For nID = TWordAddin.[_First] To TWordAddin.[_Last] Step 1
    If iNew.data_.Item(nID) Then
      If sParams <> vbNullString Then _
        sParams = sParams & ", "
      sParams = sParams & ProductNameFromID(nID) & "+"
    ElseIf iCurrent.data_.Item(nID) Then
      If sParams <> vbNullString Then _
        sParams = sParams & ", "
      sParams = sParams & ProductNameFromID(nID) & "-"
    End If
  Next nID
  
  ApplyAddins = VersionUpdateProducts(sParams)
End Function

Public Function ExecuteSearch(target As Excel.Workbook, params As ParametersSearch)
  Dim iOut As Excel.Worksheet: Set iOut = target.Sheets.Add
  Dim iEngine As New SearchEngine
  Call iEngine.Init(iOut, params)
  Call iEngine.Execute
End Function

Public Function UpdateConceptPowershell() As Boolean
  UpdateConceptPowershell = False
  
  Dim fso As New Scripting.FileSystemObject
  Dim sInstall$: sInstall = CP_TOOLS_SERVER & "\" & CONCEPT_INSTALLER
  If Not fso.FileExists(sInstall) Then _
    Exit Function
  
  On Error GoTo RETURN_FALSE
  Dim sExec$: sExec = "cmd.exe /c " & """" & sInstall & """" & " < nul"
  Dim iShell As New WshShell
  iShell.CurrentDirectory = CP_TOOLS_SERVER
  UpdateConceptPowershell = iShell.Run(sExec, waitOnReturn:=True) = 0
  Exit Function

RETURN_FALSE:
  On Error GoTo 0
End Function

Public Function OutputConceptStatus(iOut As Excel.Worksheet)
  With iOut
    .Cells(1, 1) = "ProductID"
    .Cells(1, 2) = " "
    .Cells(1, 3) = " "
    .Cells(1, 4) = " "
    .Columns(1).ColumnWidth = 25
    .Columns(2).ColumnWidth = 15
    .Columns(3).ColumnWidth = 15
    .Columns(4).ColumnWidth = 15
  End With
  
  Dim nRow&: nRow = 2
  
  Dim iLocal As API_DistrManifest: Set iLocal = VersionLocalManifest
  Dim iServer As API_DistrManifest: Set iServer = VersionServerManifest
  Dim iEnabled As AddinOptions: Set iEnabled = ScanAddinOptions
  
  Dim iProduct As Variant
  For Each iProduct In iLocal.data_.Keys
    Dim sProduct$: sProduct = VBA.CStr(iProduct)
    
    Dim sLocalVer$: sLocalVer = iLocal.GetVersion(sProduct)
    If sLocalVer = vbNullString Then _
      sLocalVer = " "
      
    Dim sServerVer$: sServerVer = iServer.GetVersion(sProduct)
    If sServerVer = vbNullString Then _
      sServerVer = " "
    
    With iOut
      .Cells(nRow, 1) = sProduct
      .Cells(nRow, 2) = sLocalVer
      .Cells(nRow, 3) = sServerVer
      .Cells(nRow, 4) = IIf(iEnabled.CheckProductEnabled(sProduct), "", "")
    End With
    nRow = nRow + 1
  Next iProduct
End Function

' =======
Private Function ScanProjects(iSource As Excel.Worksheet) As Collection
  Set ScanProjects = New Collection
  Dim nRow&
  Dim lstRow&: lstRow = iSource.Cells(iSource.Rows.Count, 1).End(xlUp).Row
  nRow = PROJECTS_FIRST_ROW
  For nRow = PROJECTS_FIRST_ROW To lstRow Step 1
    Dim sTxt$: sTxt = iSource.Cells(nRow, 1)
    If sTxt <> vbNullString Then _
      Call ScanProjects.Add(sTxt)
  Next nRow
End Function
