VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "SearchEngine"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private out_ As Excel.Worksheet
Private params_ As ParametersSearch

Private queryWords_ As Scripting.Dictionary
Private queryCount_ As Long

Private foundIDs_() As Collection
Private wordCounts_() As Long

Private Const PROGRESS_DICTIONARY = 10
Private Const PROGRESS_SEARCH = 89
Private Const PROGRESS_RESULTS = 1

Private Const MAX_COLUMNS = 250

' _SR_ - search results
Private Enum SearchResultStruct
  [_First] = 1
  
  S_SR_INPUT = 1
  S_SR_COUNT = 2
  S_SR_DATA = 3
  
  [_Last] = 3
End Enum

Public Function Init(iOut As Excel.Worksheet, params As ParametersSearch)
  Set out_ = iOut
  Set params_ = params
  If Not params_.ignoreSmall_ Then _
    params_.smallSize_ = 0
End Function

Public Function Execute()
  Call CSE_ProgressBar.Init(" ", maxVal:=100, canInterrupt:=True)
  Call CSE_ProgressBar.ShowModeless
  
  CSE_ProgressBar.Description = "   ..."
  Call PrepareIndexDictionary
  
  CSE_ProgressBar.Description = "..."
  Call ExecuteSearch
  
  CSE_ProgressBar.Description = " ..."
  Call FinalizeResults
  Call CSE_ProgressBar.IncrementA(PROGRESS_RESULTS)
  
  Call Unload(CSE_ProgressBar)
End Function

' ========
Private Function PrepareIndexDictionary()
  queryCount_ = CountNonEmpty(params_.searchWhat_)
  Call ResetWordCounts
  Call CSE_ProgressBar.InitSecondBar(maxVal:=queryCount_)
  
  Set queryWords_ = New Scripting.Dictionary
  Dim nItem&: nItem = 1
  Dim iCell As Excel.Range
  Dim iWords As Collection
  For Each iCell In params_.searchWhat_
    Set iWords = ExtractWords(iCell.Text)
    wordCounts_(nItem) = iWords.Count
    
    Dim iWord As Variant
    For Each iWord In iWords
      If params_.normalize_ Then _
        iWord = PY_Normalize(CStr(iWord))
      If Not queryWords_.Exists(CStr(iWord)) Then _
        Set queryWords_(CStr(iWord)) = New Collection
      On Error Resume Next
      Call queryWords_(CStr(iWord)).Add(nItem, CStr(nItem))
      On Error GoTo 0
    Next iWord
    
    If nItem = queryCount_ Then _
      Exit For
    nItem = nItem + 1
    Call CSE_ProgressBar.IncrementB
    Call CSE_ProgressBar.SetA(PROGRESS_DICTIONARY * nItem / queryCount_)
  Next iCell
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function ExecuteSearch()
  Dim nTextCount&: nTextCount = CountNonEmpty(params_.searchWhere_)
  Call ResetFoundIDs
  Call CSE_ProgressBar.InitSecondBar(maxVal:=nTextCount)
  
  Dim nItem&: nItem = 1
  Dim iCell As Excel.Range
  Dim iWords As Collection
  Dim iCounts As Scripting.Dictionary
  Dim vQueryID As Variant
  Dim iWord As Variant
  
  For Each iCell In params_.searchWhere_
    Set iCounts = New Scripting.Dictionary
    Set iWords = ExtractWords(iCell.Text)
    
    For Each iWord In iWords
      If params_.normalize_ Then _
        iWord = PY_Normalize(CStr(iWord))
      If queryWords_.Exists(CStr(iWord)) Then
        For Each vQueryID In queryWords_(CStr(iWord))
          If Not iCounts.Exists(CStr(vQueryID)) Then _
            iCounts(CStr(vQueryID)) = 0
          iCounts(CStr(vQueryID)) = iCounts(CStr(vQueryID)) + 1
        Next vQueryID
      End If
    Next iWord
    
    For Each vQueryID In iCounts.Keys
      If params_.partialMatch_ Or wordCounts_(CLng(vQueryID)) <= iCounts(vQueryID) Then _
        Call foundIDs_(CLng(vQueryID)).Add(nItem, CStr(nItem))
    Next vQueryID
    
    If CSE_ProgressBar.Interrupted Or nItem = nTextCount Then _
      Exit For
    nItem = nItem + 1
    Call CSE_ProgressBar.IncrementB
    Call CSE_ProgressBar.SetA(PROGRESS_DICTIONARY + PROGRESS_SEARCH * nItem / nTextCount)
  Next iCell
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function FinalizeResults()
  Call CSE_ProgressBar.InitSecondBar(maxVal:=queryCount_)
  
  Dim nItem&: nItem = 1
  Dim nCount&
  
  Dim iCell As Excel.Range
  For Each iCell In params_.searchWhat_
    nCount = foundIDs_(nItem).Count
    out_.Cells(nItem, S_SR_INPUT) = iCell
    out_.Cells(nItem, S_SR_COUNT) = foundIDs_(nItem).Count
    If nCount = 0 Then _
      GoTo NEXT_ITEM
      
    If params_.outputText_ Then
      Call OutputWords(nItem)
    Else
      out_.Cells(nItem, 3) = ListIDs(nItem)
    End If
    
NEXT_ITEM:
    If nItem = queryCount_ Then _
      Exit For
    nItem = nItem + 1
    Call CSE_ProgressBar.IncrementB
  Next iCell
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function ExtractWords(sText$) As Collection
  Dim iWords As New Collection
  Dim nStart&: nStart = 1
  Dim nLen&: nLen = VBA.Len(sText)
  Dim nCur&: nCur = nStart
  Do While nCur <= nLen
    If IsDelim(VBA.Mid$(sText, nCur, 1)) Then
      If nCur > nStart And nCur - nStart >= params_.smallSize_ Then _
        Call iWords.Add(VBA.Mid$(sText, nStart, nCur - nStart))
      nStart = nCur + 1
    End If
    nCur = nCur + 1
  Loop
  
  If nCur > nStart And nCur - nStart >= params_.smallSize_ Then _
    Call iWords.Add(VBA.Mid$(sText, nStart, nCur - nStart))
  Set ExtractWords = iWords
End Function

Private Function ResetFoundIDs()
  ReDim foundIDs_(1 To queryCount_)
  Dim nItem&
  For nItem = 1 To queryCount_ Step 1
    Set foundIDs_(nItem) = New Collection
  Next nItem
End Function

Private Function ResetWordCounts()
  ReDim wordCounts_(1 To queryCount_)
End Function

Private Function CountNonEmpty(target As Excel.Range) As Long
  Dim emptyIndex&: emptyIndex = XLFirstEmptyCell(target)
  If emptyIndex = -1 Then
    CountNonEmpty = target.Count
  Else
    CountNonEmpty = emptyIndex - 1
  End If
End Function

Private Function ListIDs(nItem&) As String
  ListIDs = ""
  Dim vID As Variant
  For Each vID In foundIDs_(nItem)
    If ListIDs <> "" Then _
      ListIDs = ListIDs & ", "
    ListIDs = ListIDs & CStr(vID)
  Next vID
End Function

Private Function OutputWords(nItem&)
  Dim nColumn&: nColumn = S_SR_DATA
  Dim vID As Variant
  For Each vID In foundIDs_(nItem)
    out_.Cells(nItem, nColumn) = params_.searchWhere_(CLng(vID))
    nColumn = nColumn + 1
    If nColumn >= MAX_COLUMNS Then _
      Exit For
  Next vID
End Function

Private Function IsDelim(sSymbol$) As Boolean
  IsDelim = True
  If sSymbol = Chr(13) Then
    Exit Function
  ElseIf sSymbol = Chr(10) Then
    Exit Function
  ElseIf sSymbol = "," Then
    Exit Function
  ElseIf sSymbol = ChrW(&H2013) Then _
    Exit Function
  ElseIf sSymbol = ChrW(&H2012) Then _
    Exit Function
  ElseIf sSymbol = ChrW(&H2010) Then _
    Exit Function
  ElseIf sSymbol = ";" Then
    Exit Function
  ElseIf sSymbol = " " Then
    Exit Function
  ElseIf sSymbol = "!" Then
    Exit Function
  ElseIf sSymbol = ":" Then
    Exit Function
  ElseIf sSymbol = "?" Then
    Exit Function
  ElseIf sSymbol = """" Then
    Exit Function
  ElseIf sSymbol = ")" Then
    Exit Function
  ElseIf sSymbol = "(" Then
    Exit Function
  ElseIf sSymbol = "Chr(85)" Then
    Exit Function
  End If
  IsDelim = False
End Function


